close all;
clear all;

%% Read the initial image and define graph creation parameters

% Sample image file
filename = 'eight.tif';
fileformat = 'tif';

input = imread(filename, fileformat);
image = im2double(input);

[height, width] = size(image);

%% Create a graph through WATERSHED segmentation and define grayscale image function on it

[~, I, indices, ~, al, c, degs, tri, n] = graph_watershed_delaunay(image);

%% Initial contour: circle

contour_internal = (sqrt((c(:, 1)-0.5*width).^2 + (c(:, 2)-0.5*height).^2) <...
    0.45*min(width, height));

%% Global parameters of the algorithm

% Gradient and curvature filters parameter
fil = [2, 2];

% Curvature approximation parameter
curv_approx = 0;

% Parameters of g function computation
sigma = 0.01*min(width, height);
lambda = 0.07;

% Total iterations parameter
iter = 4000;

% Time evolution parameter
dt = 0.005;

% Balloon force regulation parameter
beta = -2;

%% Initialization of the algorithm on the graph

% Compute the fundamental geometric quantities of the graph essential for
% gradient and curvature calculations
[edgevectors, lengths, right_lengths, angles, omegas, index] =...
    edgevectors_angles(n, al, c(:, 1:2), degs);

% Initialize the GAC algorithm
[g, g_grmag, g_grdir, Phi] = gac_init(I, c(:, 1:2), sigma, lambda, contour_internal,...
    n, al, degs, edgevectors, lengths, angles, fil(1));

% Modify signed distance transform so that the initial contour surrounds
% all the objects and the SDT has the right range of values
Phi = Phi + 0.5*max(width, height) - 0.45*min(width, height);
Phi = Phi/min(width, height);

%% Main iteration stage of the algorithm

figure;
plot(c(:, 1), height - c(:, 2), 'o', 'MarkerSize', 2);
hold on;
plot(c(Phi > 0, 1), height - c(Phi > 0, 2), 'o',...
    'MarkerEdgeColor', 'k',...
    'MarkerFaceColor', 'r',...
    'MarkerSize', 3);
hold off;
axis equal;

stages = 200;

for i=1:iter
    Phi = gac_iter(Phi, g, g_grmag, g_grdir, beta, dt, edgevectors, lengths,...
        angles, omegas, right_lengths, index, n, al, degs, fil, curv_approx);
    if ~mod(i, stages)
        figure;
        plot(c(:, 1), height - c(:, 2), 'o', 'MarkerSize', 2);
        hold on;
        plot(c(Phi > 0, 1), height - c(Phi > 0, 2), 'o',...
            'MarkerEdgeColor', 'r',...
            'MarkerFaceColor', 'r',...
            'MarkerSize', 2);
        hold off;
        axis equal;
    end
end
