% Creating the graph
close all;
clear all;

n = 5500;
a = 1/3;
b = 3/5;
r = b*n^(-a);
[~, al, c, degs] = grgall_small_lists(n, r);

% Defining a Gaussian function supported on the graph's 
% vertices, which will be used to compute the graph-based approximation to
% gradient and compare it to its known analytic expression

x0 = 0.5;
y0 = 0.5;
sigma = 0.25;
u = exp(-((c(:, 1) - x0).^2 + (c(:, 2) - y0).^2)/(2*sigma^2));

% Calculation of edge vectors directions, lengths and angles  in order to
% initialize gradient approximation parameters

[edgevectors, lengths, ~, angles, ~] = edgevectors_angles(n, al, c, degs);

% Calculation of gradient approximation 

grad_u = gradient_approximation_step(n, al, degs, u,...
    edgevectors, lengths, angles);

% Average/median filtering inside neighborhood on the approximate gradient
% vector field to smooth the result

grad_u_av = gradient_average_filter(al, degs, n, grad_u);
grad_u_med = gradient_median_filter(al, degs, n, grad_u);

% Analytical expression for gradient in the trigonometric or the Gaussian
% case, used for comparison reasons

grad_anal_u = [-(c(:, 1)-x0).*u/sigma^2 -(c(:, 2)-y0).*u/sigma^2];
gradmagn_anal_u = u.*sqrt((c(:, 1) - x0).^2 + (c(:, 2) - y0).^2)/(sigma^2);

%% Gradient vector field plotting

scale = 2;

% Exact vector field
figure;
quiver(c(:, 1), c(:, 2), grad_anal_u(:, 1), grad_anal_u(:, 2), scale);
axis square;
h1 = gcf;
h1 = tightfig(h1);

% Approximate vector field without filtering
figure;
quiver(c(:, 1), c(:, 2), grad_u(:, 1), grad_u(:, 2), scale);
axis square;
h1 = gcf;
h1 = tightfig(h1);

% Approximate vector field with average filtering
figure;
quiver(c(:, 1), c(:, 2), grad_u_av(:, 1), grad_u_av(:, 2), scale);
axis square;
h1 = gcf;
h1 = tightfig(h1);

% Approximate vector field with median filtering
figure;
quiver(c(:, 1), c(:, 2), grad_u_med(:, 1), grad_u_med(:, 2), scale);
axis square;
h1 = gcf;
h1 = tightfig(h1);
