function [grad_av, gradmagn_av] = gradient_average_filter(al, degs, n, grad)
    
    % Inputs:
    %   al      - adjacency list of the graph stored as an nxm matrix,
    %           where m is maximal vertex degree on the graph
    %   degs    - 1xn vector holding the degree of each vertex
    %   n       - number of graph vertices
    %   grad    - initial gradient field approximation as an nx2 matrix

    % Outputs:
    %   grad_av     - nx2 matrix with filtered gradient field
    %   gradmagn_av - nx1 vector with magnitude of gradient field

    % Add every vertex to its adjacent list so that its gradient value is taken
    % into account in the average filtering
    degs = degs + 1;
    al = [al zeros(n, 1)];
    for i=1:n
        al(i, degs(i)) = i;
    end
    
    grad_av = zeros(n, 2);
    
    % Main average filtering loop
    for i=1:n
        grad_av(i, :) = mean(grad(al(i, 1:degs(i)), :));
    end
    
    % Calculate gradient magnitude after filtering the approximation
    gradmagn_av = sqrt(grad_av(:,1).^2 + grad_av(:, 2).^2);
        
end

